import { Request, Response } from 'express';
import { prisma, __path, log } from '../../../index.js';

// Get category.
function get(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.query.name) {
		res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
		return;
	}

	prisma.itemCategory
		.findUnique({
			where: {
				name: req.query.name.toString()
			}
		})
		.then((item) => {
			if (item) {
				res.status(200).json(JSON.stringify(item));
			} else {
				res.status(410).json({ error: 'Category does not exist.' });
			}
		})
		.catch((err) => {
			console.error(err);
			res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
		});
}

// Create category.
function post(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.name) {
		res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
		return;
	}

	// Save data.
	prisma.itemCategory
		.create({
			data: {
				name: req.body.name,
				description: req.body.description
			}
		})
		.then(() => {
			res.status(201).json({ status: 'created' });
		})
		.catch((err) => {
			// TODO Catch if is a duplicate error and show a message to the user
			log.db.error(err);
			res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
		});
}

// Update category.
function patch(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id || !req.body.name) {
		res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
		return;
	}

	prisma.itemCategory
		.update({
			where: {
				id: parseInt(req.body.id)
			},
			data: {
				name: req.body.name,
				description: req.body.description
			}
		})
		.then(() => {
			res.status(201).json({ status: 'updated' });
		})
		.catch((err) => {
			// TODO Catch if is a duplicate error and show a message to the user
			log.db.error(err);
			res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
		});
}

// Delete category.
async function del(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id) {
		res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
		return;
	}

	// Does the id exist? If not return 410 Gone.
	try {
		const result = await prisma.itemCategory.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(410).json({ error: 'Category does not exist.' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
	}

	prisma.itemCategory
		.delete({
			where: {
				id: parseInt(req.body.id)
			}
		})
		.then(() => {
			res.status(201).json({ status: 'deleted' });
		})
		.catch((err) => {
			log.db.error(err);
			res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
		});
}

export default { get, post, patch, del };
